/*
 *				**** DRAW TOOL ROUTINES ****
 *
 * This file contains the private routines
 * for the DrawTool program
 */

#include <stdio.h>
#include <stdlib.h>
#include <mem.h>

#include "drawtool.h"

/*
 * Private Data structure definitions
 */

struct draw_obj {
	struct draw_obj *next;	/* next object */
	int type;	/* type of object */
   DrawPoint *points;	/* location of object */
};

/* internal variables */
static int minx=MINX, maxx=MAXX, miny=MINY, maxy=MAXY;

/*
 * data structure for drawing types
 */
struct { /* notice no struct name is needed */
	int type;	/* drawing type */
   char *desc;	/* description */
} ObjType[] = { /* notice array has a name */
{ DRAWOBJ_PT, "Point" },
{ DRAWOBJ_LINE, "Line Segment(s)"},
{ DRAWOBJ_RECT, "Rectangle"},
{ DRAWOBJ_CIRC, "Circle"},
{ DRAWOBJ_POLY, "Polygon"},
{ DRAWOBJ_BAD, NULL}};

/* routines for object types */
char *
get_drawobj_desc(int type)
{
	int i;

   /* find entry */
   for (i=0; ObjType[i].type != DRAWOBJ_BAD && ObjType[i].type != type; i++);
	return ObjType[i].desc;
}

/* routines for points */

DrawPoint *
create_point(void)
{
	DrawPoint *ret;

   ret = (DrawPoint *)malloc(sizeof(*ret));
   if (ret != NULL) memset(ret, '\0', sizeof(*ret));
   return ret;
}

void
free_point(DrawPoint *pt)
{ /* free all points in chain */
	DrawPoint *p;

   while (pt != NULL)
   { /* free all points */
   	p = pt->next;	/* save next point */
	   memset(pt, '\0', sizeof(*pt));
   	free(pt);
      pt = p;	/* next point */
   }
}

void
point_setx(DrawPoint *pt, int x)
{
	if (x < minx || x > maxx) return; /* bad args */
   pt->x = x;
}

void
point_sety(DrawPoint *pt, int y)
{
	if (y < miny || y > maxy) return; /* bad args */
   pt->y = y;
}

void
point_setxy(DrawPoint *obj, int x, int y)
{
	point_setx(obj, x);
   point_sety(obj, y);
}

DrawObj *
create_drawobj(void)
{
	DrawObj *ret;

   ret = (DrawObj *)malloc(sizeof(*ret));
   if (ret != NULL) memset(ret, '\0', sizeof(*ret));
   return ret;
}

void
free_drawobj(DrawObj *obj)
{ /* routine has memory leak */
	if (obj == NULL) return;
   free_point(obj->points);
   memset(obj, '\0', sizeof(*obj));
   free(obj); /* free memory */
}

void
drawobj_set_type(DrawObj *obj, int type)
{
	if (obj == NULL || get_drawobj_desc(type) == NULL) return;
   obj->type = type;	/* valid type */
}

void
drawobj_set_point(DrawObj *obj, DrawPoint *pt)
{
	if (obj == NULL || pt == NULL) return;
	obj->points = pt;
}

void
drawobj_draw(DrawObj *obj)
{
	DrawPoint *pt;
   char *p;

   for (; obj != NULL; obj = obj->next)
	{ /* loop through objects */
		p = get_drawobj_desc(obj->type);
      if (p == NULL) p = "???";
      printf("Draw object type %d (%s) at location", obj->type, p);
      /* what does this next statement do? */
      if (obj->points->next == NULL) printf(" ");
      else printf("s ");
      for (pt = obj->points; pt != NULL; pt = pt->next)
      { /* loop thru points */
      	if (pt != obj->points)
         { /* what does this do? */
         	printf(", ");
         	if (pt->next == NULL) printf("and ");
         }
			printf("(%d, %d)", pt->x, pt->y);
      }
      printf("\n");
   }
}
