/*
 *			**** DRAW TOOL ****
 *
 * This is the public header file for DrawTool
 */

 /**************** Definitions ****************/

/* default limits on drawing space */
#define MINX	0		/* smallest X value */
#define MAXX	100	/* largest X value */
#define MINY	0		/* smallest Y value */
#define MAXY	100	/* largest Y value */

#define DRAWOBJ_BAD		-1	/* invalid obj */
#define DRAWOBJ_PT		1	/* single point */
#define DRAWOBJ_LINE		2	/* line segment(s) */
#define DRAWOBJ_RECT		3	/* rectangle */
#define DRAWOBJ_CIRC		4	/* circle */
#define DRAWOBJ_POLY		5	/* polygon */

 /*************** Dawing Objects **************/

/* public object that is visible to client routines */
/* point in 2-D space */
typedef struct point {
	struct point *next;	/* next point (if any) */
	int x;	/* x coordinate */
   int y;	/* y coordinate */
} DrawPoint;

/* constructor * destructor */
DrawPoint *create_point(void);
void free_point(DrawPoint *pt);

/* routines to set coordinates */
void point_setx(DrawPoint *pt, int x);
void point_sety(DrawPoint *pt, int y);
void point_setxy(DrawPoint *pt, int x, int y);

/* private object - not visible by client routines */
/* generic drawing object */
typedef struct draw_obj DrawObj;

/*
 *				**** DRAW OBJ ****
 *
 * generic drawing object routines
 */

/* constructor & destructor */
DrawObj *create_drawobj(void); /* constructor */
void free_drawobj(DrawObj *obj); /* destructor */

/* type of drawing object */
void drawobj_set_type(DrawObj *obj, int t);
int drawobj_get_type(DrawObj *obj);

/* point(s) associated with drawing object */
DrawPoint *drawobj_get_point(DrawObj *obj);
void drawobj_set_point(DrawObj *obj, DrawPoint *pt);

/* output drawing */
void drawobj_draw(DrawObj *obj);

